using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;

using CMS.Ecommerce;
using CMS.TreeEngine;
using CMS.CMSHelper;
using CMS.FormEngine;
using CMS.IDataConnectionLibrary;
using CMS.SettingsProvider;
using CMS.GlobalHelper;
using CMS.DataEngine;
using CMS.SiteProvider;
using CMS.DirectoryUtilities;

public partial class ImportProducts : System.Web.UI.Page
{

    protected void Page_Load(object sender, EventArgs e)
    {        
        lblError.Text = Import(Server.MapPath("products.csv"));
    }

    /// <summary>
    /// Imports products from the specified file in *.csv format
    /// </summary>
    /// <param name="filePath"></param>
    /// <returns></returns>
    private string Import(string filePath)
    {
        string error = "";
        FileStream file = null;

        try
        {
            // Specify file with product data
            file = new FileStream(filePath, FileMode.Open, FileAccess.Read);
        }
        catch (Exception ex)
        {
            error = "Error loading file '" + filePath + "': " + ex.Message;
        }


        if (error == "")
        {
            // Get product default department
            DepartmentInfo defaultDepartment = GetDefaultDepartment();

            // Get product parent document
            CMS.TreeEngine.TreeNode parentDocument = GetProductParentDocument();

            // Get default product document type
            DataClassInfo productDocumentType = GetProductDocumentType();

            if ((defaultDepartment == null) || (parentDocument == null) || (productDocumentType == null))
            {
                error = "Unable to create products, some information missing";
            }
            else
            {
                StreamReader reader = new StreamReader(file);
                int count = 0;

                // Read file
                while (!reader.EndOfStream)
                {
                    string line = reader.ReadLine();
                    if (line.Trim() != "")
                    {
                        // Get product data
                        string[] productData = line.Trim().Split(',');

                        // Creat product (SKU)
                        SKUInfo sku = new SKUInfo();
                        sku.SKUName = productData[0];
                        sku.SKUDescription = productData[1];
                        sku.SKUPrice = ValidationHelper.GetDouble(productData[2], 0);
                        sku.SKUEnabled = true;

                        DepartmentInfo department = DepartmentInfoProvider.GetDepartmentInfo(productData[3]);
                        if (department != null)
                        {
                            // Assign product to its department
                            sku.SKUDepartmentID = department.DepartmentID;
                        }
                        else
                        {
                            // Assign product to the default department
                            sku.SKUDepartmentID = defaultDepartment.DepartmentID;
                        }

                        try
                        {
                            SKUInfoProvider.SetSKUInfo(sku);
                        }
                        catch
                        {
                            error += "Unable to create product '" + sku.SKUName + "'. <br />";
                        }

                        // If product was create succesfully
                        if (sku.SKUID > 0)
                        {

                            // Create product document type and assign SKU to it
                            CMS.TreeEngine.TreeNode productDoc = new CMS.TreeEngine.TreeNode("custom.defaultproduct");
                            productDoc.SetValue("ProductName", sku.SKUName);
                            productDoc.DocumentName = sku.SKUName;
                            productDoc.NodeSKUID = sku.SKUID;
                            productDoc.DocumentCulture = CMSContext.CurrentUser.PreferredCultureCode;

                            try
                            {
                                productDoc.Insert(parentDocument.NodeID);
                                count++;
                            }
                            catch
                            {
                                error += "Unable to create document '" + sku.SKUName + "'. <br />";
                                SKUInfoProvider.DeleteSKUInfo(sku.SKUID);
                            }
                        }

                    }
                }

                // Close file
                file.Close();

                // Close reader
                reader.Close();

                // Display number of created products
                lblInfo.Text = "Number of created products: " + count;
            }
        }

        return error;
    }


    /// <summary>
    /// Ensures default department exists.
    /// </summary>    
    private DepartmentInfo GetDefaultDepartment()
    {
        // Try to get default department    
        DepartmentInfo defaultDepartment = DepartmentInfoProvider.GetDepartmentInfo("DefaultDepartment");

        // If default department doesnt exist, create new one
        if (defaultDepartment == null)
        {
            defaultDepartment = new DepartmentInfo();
            defaultDepartment.DepartmentDisplayName = "Default department";
            defaultDepartment.DepartmentName = "DefaultDepartment";
            defaultDepartment.DepartmentDefaultTaxClassID = 0;
            DepartmentInfoProvider.SetDepartmentInfo(defaultDepartment);
        }

        return defaultDepartment;
    }


    /// <summary>
    /// Ensures parent document for product documents exists.
    /// </summary>
    private CMS.TreeEngine.TreeNode GetProductParentDocument()
    {

        CMS.TreeEngine.TreeNode parent = null;
        TreeProvider tree = new TreeProvider();

        // Try to get products' parent document
        parent = tree.SelectSingleNode(CMSContext.CurrentSiteName, "/Custom-Products", TreeProvider.ALL_CULTURES, true, "cms.menuitem");

        // Parent not found
        if (parent == null)
        {
            // Get site root
            CMS.TreeEngine.TreeNode root = tree.SelectSingleNode(CMSContext.CurrentSiteName, "/", TreeProvider.ALL_CULTURES, true, "cms.root");
            if (root != null)
            {
                // Create new parent document
                parent = new CMS.TreeEngine.TreeNode("cms.menuitem");
                parent.SetValue("MenuItemName", "Custom products");
                parent.DocumentName = "Custom products";
                parent.DocumentCulture = CMSContext.CurrentUser.PreferredCultureCode;
                parent.Insert(root.NodeID);
            }
        }

        return parent;
    }


    /// <summary>
    /// Ensures default product document type exists.
    /// </summary>    
    private DataClassInfo GetProductDocumentType()
    {
        // Try to get default product document type
        string className = "custom.defaultproduct";
        DataClassInfo defaultProductType = DataClassInfoProvider.GetDataClass(className);

        // Default product document type not found -> create new one
        if (defaultProductType == null)
        {

            string tableName = "Custom_DefaultProduct";
            string primaryKey = "ProductID";

            // Initialize new product type
            defaultProductType = new DataClassInfo();
            defaultProductType.ClassDisplayName = "Default product";
            defaultProductType.ClassName = "custom.defaultproduct";
            defaultProductType.ClassIsDocumentType = true;
            defaultProductType.ClassUsePublishFromTo = true;
            defaultProductType.ClassTableName = tableName;

            try
            {
                // Save document type 
                DataClassInfoProvider.SetDataClass(defaultProductType);

                // Create document type table
                TableManager.CreateTable(tableName, primaryKey);

                // Add table columns                
                TableManager.AddTableColumn(tableName, "ProductName", "nvarchar(200)", false, null);

                // Create document type form definition
                FormInfo form = new FormInfo("<form></form>");
                FormFieldInfo field = new FormFieldInfo();

                // Create primary key field
                field.Name = primaryKey;
                field.Caption = primaryKey;
                field.DataType = FormFieldDataTypeEnum.Integer;
                field.DefaultValue = "";
                field.Description = "";
                field.FieldType = FormFieldControlTypeEnum.LabelControl;
                field.PrimaryKey = true;
                field.System = false;
                field.Visible = true;
                field.Size = 0;
                field.AllowEmpty = false;

                // Add primary key field
                form.AddFormField(field);

                // Create product name field
                field.Name = "ProductName";
                field.Caption = "Product name";
                field.DataType = FormFieldDataTypeEnum.Text;
                field.DefaultValue = "";
                field.Description = "";
                field.FieldType = FormFieldControlTypeEnum.TextBoxControl;
                field.PrimaryKey = false;
                field.System = false;
                field.Visible = true;
                field.Size = 200;
                field.AllowEmpty = false;

                // Add product name field
                form.AddFormField(field);

                // Update document type
                defaultProductType.ClassXmlSchema = TableManager.GetXmlSchema(tableName);
                defaultProductType.ClassFormDefinition = form.GetXmlDefinition();
                defaultProductType.ClassNodeNameSource = "ProductName";
                defaultProductType.ClassIsCoupledClass = true;
                DataClassInfoProvider.SetDataClass(defaultProductType);


                // Ensure default permissions
                PermissionNameInfoProvider.CreateDefaultClassPermissions(defaultProductType.ClassID);

                // Generate basic queries
                SqlGenerator.GenerateQuery(className, "select", SqlOperationTypeEnum.SelectQuery, false);
                SqlGenerator.GenerateQuery(className, "delete", SqlOperationTypeEnum.DeleteQuery, false);
                SqlGenerator.GenerateQuery(className, "insert", SqlOperationTypeEnum.InsertQuery, true);
                SqlGenerator.GenerateQuery(className, "update", SqlOperationTypeEnum.UpdateQuery, false);
                SqlGenerator.GenerateQuery(className, "selectall", SqlOperationTypeEnum.SelectAll, false);
                SqlGenerator.GenerateQuery(className, "searchtree", SqlOperationTypeEnum.SearchTree, false);
                SqlGenerator.GenerateQuery(className, "selectdocuments", SqlOperationTypeEnum.SelectDocuments, false);
                SqlGenerator.GenerateQuery(className, "selectversions", SqlOperationTypeEnum.SelectVersions, false);


                // Allow product type to be inserted as a child of the menu item type
                DataClassInfo parentClass = DataClassInfoProvider.GetDataClass("cms.menuitem");
                if (parentClass != null)
                {
                    DataClassInfoProvider.AllowChildClass(parentClass.ClassID, defaultProductType.ClassID);
                }

                // Add product type to site
                DataClassInfoProvider.AddClassToSite(defaultProductType.ClassID, CMSContext.CurrentSite.SiteID);


                // Create product default transformation
                TransformationInfo ti = new TransformationInfo();
                ti.TransformationName = ResHelper.GetString("TransformationName.Default");
                ti.TransformationFullName = className + "." + ti.TransformationName;
                ti.TransformationCode = TransformationInfoProvider.GenerateTransformationCode(defaultProductType.ClassFormDefinition, TransformationTypeEnum.Ascx, className);
                ti.TransformationType = TransformationTypeEnum.Ascx;
                ti.TransformationClassID = defaultProductType.ClassID;
                TransformationInfoProvider.SetTransformation(ti);

                // Document type icon
                try
                {
                    // Get default icon path
                    string sourceFile = "~/App_Themes/" + this.Theme + "/Images/CMSDesk/Icons/default.gif";
                    
                    // Get document type icon path
                    string destFile = "~/App_Themes/" + this.Theme + "/Images/CMSDesk/Icons/" + className.Replace(".", "_") + ".gif";

                    // Create new document type icon via copying default icon    
                    File.Copy(Server.MapPath(sourceFile), Server.MapPath(destFile), false);
                }
                catch
                {
                }

            }
            // Error while creating default product document type
            catch
            {
                // Remove all product type data
                DataClassInfoProvider.DeleteDataClass(className);
                return null;
            }
        }

        return defaultProductType;
    }
}
